#include <Wire.h>
#include <SPI.h>
#include "MAX30105.h"
#include "heartRate.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Servo.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

MAX30105 particleSensor;
long lastBeat = 0;
float beatsPerMinute;
float beatAvg;
float last_beatAvg;
int beat_counter = 0;
int lvl_puls = 0;
int last_lvl_puls = 0;
bool first_run = true;
bool finger_erkannt = false;

int StepDown_cs = 9;
Servo servo;

int Herz_Switch = 2;
int Herz_Signal = 8;

int Herz_Relay = 4;

void setup() {
  Serial.begin(9600);

  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println("OLED nicht erkannt");
    for (;;);
  }
  display.clearDisplay();

  if (!particleSensor.begin(Wire, I2C_SPEED_FAST)) //Use default I2C port, 400kHz speed
  {
    Serial.println("Pulssensor nicht erkannt");
    while (1);
  }

  particleSensor.setup();
  particleSensor.setPulseAmplitudeRed(100); //Turn Red LED to low to indicate sensor is running
  particleSensor.setPulseAmplitudeGreen(0); //Turn off Green LED

  SPI.begin();
  pinMode(StepDown_cs, OUTPUT);
  digitalWrite(StepDown_cs, LOW);
  
  servo.attach(7);
  pinMode(Herz_Switch, INPUT);

  for (int i = 20; i > 0; i--) {
    SPI.transfer(0);
    delay(120);
  }
  servo.write(20);
  pinMode(Herz_Signal, OUTPUT);
  pinMode(Herz_Relay, OUTPUT);
}

void getPulse() {
  long irValue = particleSensor.getIR();

  if (checkForBeat(irValue) == true)
  {
    long delta = millis() - lastBeat;
    lastBeat = millis();

    beatsPerMinute = 60 / (delta / 1000.0);
    if (beatsPerMinute > 30 && beatsPerMinute < 120) {
      if (beat_counter < 5) {
        beatAvg = beatAvg + beatsPerMinute;
        Serial.println(beatsPerMinute);
        beat_counter++;
      } else {
        beatAvg = beatAvg / 5;
        if ((beatAvg > last_beatAvg + 5 or beatAvg < last_beatAvg - 5) && first_run == false) {
          last_beatAvg = last_beatAvg;
        } else {
          last_beatAvg = beatAvg;
          first_run = false;
        }
        beat_counter = 0;
        beatAvg = 0;
        Serial.print("Durchschnitt: ");
        Serial.println(last_beatAvg);
        finger_erkannt = true;
      }
    }
  }

  if (irValue < 50000) {
    Serial.println("Kein Finger erkannt!");
    //finger_erkannt = false;
    beatAvg = 0;
    beat_counter = 0;

    last_beatAvg = 66; //damit bei der präsenattion das herz sich bewegt
    finger_erkannt = true;
    
    delay(400);
  }
}

void printText (String Puls, int lvl_speed) {
  display.clearDisplay();

  display.setTextSize(4);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 2);
  display.print(Puls);

  display.setTextSize(2);
  display.setTextColor(SSD1306_WHITE);
  display.println(" BPM           " + String(lvl_speed));

  display.display();
}

void set_geschwindigkeit(int speed_lvl) {
  int Switch_Status = digitalRead(Herz_Switch);
  
  if (Switch_Status == 0) {
    int SPI_speed = speed_lvl * 12;
    if (last_lvl_puls > SPI_speed) {
      for (int i = SPI_speed; i < last_lvl_puls; i--) {
        SPI.transfer(0);
        SPI.transfer(i);
        delay(120);
      }
    } else if (last_lvl_puls < SPI_speed) {
      for (int i = last_lvl_puls; i < SPI_speed; i++) {
        SPI.transfer(0);
        SPI.transfer(i);
        delay(120);
      }
    } else {
      for (int i = 0; i < 10; i++) {
        SPI.transfer(0);
        SPI.transfer(SPI_speed);
        delay(120);
      }
    }
    last_lvl_puls = SPI_speed;
  }
}

void Main() {
  /*
     Pulsabschnitte
     50 - 65 BPM = Stufe 1
     66 - 75 BPM = Stufe 2
     76 - 85 BPM = Stufe 3
     86 - 95 BPM = Stufe 4
  */

  getPulse();
  if (finger_erkannt) {
    int beat = int(last_beatAvg);
    if (beat >= 50 && beat <= 65) {
      lvl_puls = 1;
    } else if (beat > 65 && beat <= 75) {
      lvl_puls = 2;
    } else if (beat > 75 && beat <= 85) {
      lvl_puls = 3;
    } else if (beat > 85 && beat <= 95) {
      lvl_puls = 4;
    } else {
      lvl_puls = 0;
    }
    printText(String(int(last_beatAvg)), lvl_puls);
    if (lvl_puls != 0) {
      set_geschwindigkeit(lvl_puls);
    }
    finger_erkannt = false;
  } else {
    printText("NA", lvl_puls);
  }

  int Switch_Status = digitalRead(Herz_Switch);
  if (Switch_Status == 1) {
    for (int i = 0; i < 10; i++) {
      SPI.transfer(0);
      delay(120);
    }
    digitalWrite(Herz_Signal, LOW);
    digitalWrite(Herz_Relay, LOW);
    Serial.println("Herz inaktiv");
  } else {
    digitalWrite(Herz_Signal, HIGH);
    digitalWrite(Herz_Relay, HIGH);
    Serial.println("Herz aktiv");
  }
}

void loop() {
  Main();
  //servo.write(120);
}
